"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.contextEnv = void 0;
const config_1 = require("./config");
const utils_1 = require("./utils");
const Web3 = require('web3');
const flarejs_1 = require("@flarenetwork/flarejs");
const flarejs_2 = require("@flarenetwork/flarejs");
const utils_2 = require("@flarenetwork/flarejs/dist/utils");
function contextEnv(path, network) {
    require('dotenv').config({ path: path });
    const config = getConfig(network);
    const privkHex = process.env.PRIVATE_KEY_HEX;
    const privkCB58 = process.env.PRIVATE_KEY_CB58;
    return context(config, privkHex, privkCB58);
}
exports.contextEnv = contextEnv;
function getConfig(network) {
    let networkConfig;
    if (network == 'flare' || network === undefined) {
        networkConfig = config_1.flare;
    }
    else if (network == 'costwo') {
        networkConfig = config_1.costwo;
    }
    else if (network == 'localflare') {
        networkConfig = config_1.localflare;
    }
    else
        throw Error('Invalid network');
    return networkConfig;
}
function context(config, privkHex, privkCB58) {
    const { protocol, ip, port, networkID, hrp } = config;
    // derive private key in both cb58 and hex if only one is provided
    const bintools = flarejs_1.BinTools.getInstance();
    if (privkHex !== undefined && privkHex !== '') {
        privkHex = utils_1.unPrefix0x(privkHex);
        const privkBuf = bintools.addChecksum(flarejs_1.Buffer.from(privkHex, 'hex'));
        privkCB58 = bintools.bufferToB58(privkBuf);
    }
    else if (privkCB58 !== undefined && privkCB58 !== '') {
        const privkBuf = bintools.cb58Decode(privkCB58);
        privkHex = privkBuf.toString('hex');
    }
    const path = '/ext/bc/C/rpc';
    const iport = port ? `${ip}:${port}` : `${ip}`;
    const rpcurl = `${protocol}://${iport}`;
    const web3 = new Web3(`${rpcurl}${path}`);
    const avalanche = new flarejs_2.Avalanche(ip, port, protocol, networkID);
    const xchain = avalanche.XChain();
    const cchain = avalanche.CChain();
    const pchain = avalanche.PChain();
    const xKeychain = xchain.keyChain();
    const cKeychain = cchain.keyChain();
    const pKeychain = pchain.keyChain();
    if (privkCB58) {
        const privKey = `${utils_2.PrivateKeyPrefix}${privkCB58}`;
        xKeychain.importKey(privKey);
        cKeychain.importKey(privKey);
        pKeychain.importKey(privKey);
    }
    const pAddressStrings = pchain.keyChain().getAddressStrings();
    const cAddressStrings = cchain.keyChain().getAddressStrings();
    const xAddressStrings = xchain.keyChain().getAddressStrings();
    const pAddressBech32 = pAddressStrings[0];
    const cAddressBech32 = cAddressStrings[0];
    const xAddressBech32 = xAddressStrings[0];
    let cAddressHex = "";
    if (privkHex) {
        const cAccount = web3.eth.accounts.privateKeyToAccount(privkHex);
        cAddressHex = cAccount.address.toLowerCase();
    }
    const xChainBlockchainID = utils_2.Defaults.network[networkID].X.blockchainID;
    const pChainBlockchainID = utils_2.Defaults.network[networkID].P.blockchainID;
    const cChainBlockchainID = utils_2.Defaults.network[networkID].C.blockchainID;
    const avaxAssetID = utils_2.Defaults.network[networkID].P.avaxAssetID;
    return {
        privkHex: privkHex,
        privkCB58: privkCB58,
        rpcurl: rpcurl,
        web3: web3,
        avalanche: avalanche,
        xchain: xchain,
        cchain: cchain,
        pchain: pchain,
        xKeychain: xKeychain,
        cKeychain: cKeychain,
        pKeychain: pKeychain,
        xAddressBech32: xAddressBech32,
        pAddressBech32: pAddressBech32,
        cAddressBech32: cAddressBech32,
        cAddressHex: cAddressHex,
        xChainBlockchainID: xChainBlockchainID,
        cChainBlockchainID: cChainBlockchainID,
        pChainBlockchainID: pChainBlockchainID,
        avaxAssetID: avaxAssetID,
        config: config
    };
}
