"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.importTxPC = exports.exportTxCP = void 0;
const utils_1 = require("./utils");
const dist_1 = require("@flarenetwork/flarejs/dist");
const utils_2 = require("@flarenetwork/flarejs/dist/utils");
/**
 * Exports funds from C-chain to P-chain
 * @param ctx - context with constants initialized from user keys
 * @param amount - amount to export from C-chain to P-chain
 * @param fee - export transaction fee
 */
function exportTxCP(ctx, amount, fee) {
    return __awaiter(this, void 0, void 0, function* () {
        const threshold = 1;
        const txcount = yield ctx.web3.eth.getTransactionCount(ctx.cAddressHex);
        const nonce = txcount;
        const locktime = new dist_1.BN(0);
        const importFee = ctx.pchain.getDefaultTxFee();
        const baseFeeResponse = yield ctx.cchain.getBaseFee();
        const baseFee = new dist_1.BN(parseInt(baseFeeResponse, 16) / 1e9);
        let unsignedTx = yield ctx.cchain.buildExportTx(amount.add(importFee), ctx.avaxAssetID, ctx.pChainBlockchainID, ctx.cAddressHex, ctx.cAddressBech32, [ctx.pAddressBech32], nonce, locktime, threshold, baseFee);
        if (fee === undefined) {
            const exportCost = utils_2.costExportTx(unsignedTx);
            fee = baseFee.mul(new dist_1.BN(exportCost));
            unsignedTx = yield ctx.cchain.buildExportTx(amount.add(importFee), ctx.avaxAssetID, ctx.pChainBlockchainID, ctx.cAddressHex, ctx.cAddressBech32, [ctx.pAddressBech32], nonce, locktime, threshold, fee);
        }
        const tx = unsignedTx.sign(ctx.cKeychain);
        const txid = yield ctx.cchain.issueTx(tx);
        // const txstatus = await cchain.getAtomicTxStatus(txid)
        const usedFee = utils_1.integerToDecimal(fee.toString(), 9);
        return { txid: txid, usedFee: usedFee };
    });
}
exports.exportTxCP = exportTxCP;
/**
 * Import funds exported from P-chain to C-chain to C-chain.
 * @param ctx - context with constants initialized from user keys
 * @param fee - import transaction fee
 */
function importTxPC(ctx, fee) {
    return __awaiter(this, void 0, void 0, function* () {
        const baseFeeResponse = yield ctx.cchain.getBaseFee();
        const baseFee = new dist_1.BN(parseInt(baseFeeResponse, 16) / 1e9);
        const evmUTXOResponse = yield ctx.cchain.getUTXOs([ctx.cAddressBech32], ctx.pChainBlockchainID);
        const utxoSet = evmUTXOResponse.utxos;
        let unsignedTx = yield ctx.cchain.buildImportTx(utxoSet, ctx.cAddressHex, [ctx.cAddressBech32], ctx.pChainBlockchainID, [ctx.cAddressBech32], baseFee);
        if (fee === undefined) {
            const importCost = utils_2.costImportTx(unsignedTx);
            fee = baseFee.mul(new dist_1.BN(importCost));
            unsignedTx = yield ctx.cchain.buildImportTx(utxoSet, ctx.cAddressHex, [ctx.cAddressBech32], ctx.pChainBlockchainID, [ctx.cAddressBech32], fee);
        }
        const tx = unsignedTx.sign(ctx.cKeychain);
        const txid = yield ctx.cchain.issueTx(tx);
        const usedFee = utils_1.integerToDecimal(fee.toString(), 9);
        return { txid: txid, usedFee: usedFee };
    });
}
exports.importTxPC = importTxPC;
