"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.exportTxPC = exports.importTxCP = void 0;
const dist_1 = require("@flarenetwork/flarejs/dist");
const utils_1 = require("@flarenetwork/flarejs/dist//utils");
/**
 * Import funds exported from C-chain to P-chain to P-chain
 * @param ctx - context with constants initialized from user keys
 */
function importTxCP(ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        const threshold = 1;
        const locktime = new dist_1.BN(0);
        const memo = dist_1.Buffer.from('PlatformVM utility method buildImportTx to import AVAX to the P-Chain from the C-Chain');
        const asOf = utils_1.UnixNow();
        const platformVMUTXOResponse = yield ctx.pchain.getUTXOs([ctx.pAddressBech32], ctx.cChainBlockchainID);
        const utxoSet = platformVMUTXOResponse.utxos;
        const unsignedTx = yield ctx.pchain.buildImportTx(utxoSet, [ctx.pAddressBech32], ctx.cChainBlockchainID, [ctx.pAddressBech32], [ctx.pAddressBech32], [ctx.pAddressBech32], memo, asOf, locktime, threshold);
        const tx = unsignedTx.sign(ctx.pKeychain);
        const txid = yield ctx.pchain.issueTx(tx);
        return { txid: txid };
    });
}
exports.importTxCP = importTxCP;
/**
 * Export funds from P-chain to C-chain.
 * @param ctx - context with constants initialized from user keys
 * @param amount - amount to export (if left undefined, it exports all funds on P-chain)
 */
function exportTxPC(ctx, amount) {
    return __awaiter(this, void 0, void 0, function* () {
        const threshold = 1;
        const locktime = new dist_1.BN(0);
        const memo = dist_1.Buffer.from("PlatformVM utility method buildExportTx to export AVAX from the P-Chain to the C-Chain");
        const asOf = utils_1.UnixNow();
        const platformVMUTXOResponse = yield ctx.pchain.getUTXOs([ctx.pAddressBech32]);
        const utxoSet = platformVMUTXOResponse.utxos;
        const fee = ctx.pchain.getDefaultTxFee();
        if (amount === undefined) {
            const getBalanceResponse = yield ctx.pchain.getBalance(ctx.pAddressBech32);
            const unlocked = new dist_1.BN(getBalanceResponse.unlocked);
            amount = unlocked.sub(fee);
        }
        const unsignedTx = yield ctx.pchain.buildExportTx(utxoSet, amount, ctx.cChainBlockchainID, [ctx.cAddressBech32], [ctx.pAddressBech32], [ctx.pAddressBech32], memo, asOf, locktime, threshold);
        const tx = unsignedTx.sign(ctx.pKeychain);
        const txid = yield ctx.pchain.issueTx(tx);
        return { txid: txid };
    });
}
exports.exportTxPC = exportTxPC;
