"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.integerToDecimal = exports.decimalToInteger = exports.formatBech32 = exports.toValidatorConfigHash = exports.parseToID = exports.unPrefix0x = exports.sleepms = void 0;
const bech32_1 = require("bech32");
const sha256 = __importStar(require("fast-sha256"));
const flarejs_1 = require("@flarenetwork/flarejs");
function sleepms(milliseconds) {
    return __awaiter(this, void 0, void 0, function* () {
        yield new Promise((resolve) => {
            setTimeout(() => {
                resolve();
            }, milliseconds);
        });
    });
}
exports.sleepms = sleepms;
function unPrefix0x(tx) {
    if (!tx) {
        return '0x0';
    }
    return tx.startsWith('0x') ? tx.slice(2) : tx;
}
exports.unPrefix0x = unPrefix0x;
const addressSep = '-';
const ripemd160Size = 20;
function parse(addrStr) {
    const addressParts = addrStr.split(addressSep);
    if (addressParts.length < 2) {
        throw new Error('no separator found in address');
    }
    const chainID = addressParts[0];
    const rawAddr = addressParts[1];
    const { hrp, addr } = parseBech32(rawAddr);
    return { chainID, hrp, addr };
}
function parseBech32(addrStr) {
    const decodeRes = bech32_1.bech32.decode(addrStr);
    if (!decodeRes)
        throw new Error('error decoding');
    const addrBytes = bech32_1.bech32.fromWords(decodeRes.words);
    return { hrp: decodeRes.prefix, addr: addrBytes };
}
function toShortId(addrBytes) {
    if (addrBytes.length !== ripemd160Size)
        throw new Error(`expected ${ripemd160Size} bytes but got ${addrBytes.length}`);
    return addrBytes;
}
function parseToIDBuffer(addrStr) {
    const { addr } = parse(addrStr);
    return toShortId(addr);
}
/**
 * Converts bech32 address to associated secp256k1 public key
 * @param addrStr - address in bech32 format
 * @returns secp256k1 public key
 */
function parseToID(addrStr) {
    const bintools = flarejs_1.BinTools.getInstance();
    const bufferID = flarejs_1.Buffer.from(parseToIDBuffer(addrStr));
    return bintools.cb58Encode(bufferID);
}
exports.parseToID = parseToID;
/**
 * Produces the configuration hash, encoding the validator configuration required by network nodes
 * @param networkID - id of the network (e.g. for localflare it is "162")
 * @param pChainPublicKey - secp256k1 public key (e.g."6Y3kysjF9jnHnYkdS9yGAuoHyae2eNmeV")
 * @param nodeID - id of the node associated with the validator
 * (e.g. "NodeID-MFrZFVCXPv5iCn6M9K6XduxGTYp891xXZ" = parseToID("P-localflare18jma8ppw3nhx5r4ap8clazz0dps7rv5uj3gy4v"))
 * @param weight - staking amount (e.g. "10000000000000")
 * @param duration - duration of the validation process in seconds (e.g. "1512000")
 * @returns Hash generated via sha-256 function
 */
function toValidatorConfigHash(networkID, pChainPublicKey, nodeID, weight, duration) {
    const enc = new TextEncoder(); // always utf-8
    const salt = 'flare' + networkID + '-';
    const pChainPublicKeyHash = sha256.hash(enc.encode(salt + pChainPublicKey));
    const nodeIDHash = sha256.hash(enc.encode(salt + nodeID));
    const nodeWeightHash = sha256.hash(enc.encode(salt + weight));
    const nodeDurationHash = sha256.hash(enc.encode(salt + duration));
    let validatorConfig = new Uint8Array(pChainPublicKeyHash.length +
        nodeIDHash.length +
        nodeWeightHash.length +
        nodeDurationHash.length);
    validatorConfig.set(pChainPublicKeyHash);
    validatorConfig.set(nodeIDHash, pChainPublicKeyHash.length);
    validatorConfig.set(nodeWeightHash, pChainPublicKeyHash.length + nodeIDHash.length);
    validatorConfig.set(nodeDurationHash, pChainPublicKeyHash.length + nodeIDHash.length + nodeWeightHash.length);
    const validatorConfigHash = sha256.hash(validatorConfig);
    return flarejs_1.Buffer.from(validatorConfigHash).toString('hex');
}
exports.toValidatorConfigHash = toValidatorConfigHash;
function formatBech32(hrp, address) {
    const bintools = flarejs_1.BinTools.getInstance();
    const buff = bintools.b58ToBuffer(address);
    const words = bech32_1.bech32.toWords([...buff].slice(0, -4));
    return bech32_1.bech32.encode(hrp, words);
}
exports.formatBech32 = formatBech32;
function decimalToInteger(dec, n) {
    let ret = dec;
    if (ret.includes('.')) {
        const split = ret.split('.');
        ret = split[0] + split[1].slice(0, n).padEnd(n, '0');
    }
    else {
        ret = ret + '0'.repeat(n);
    }
    return ret;
}
exports.decimalToInteger = decimalToInteger;
function integerToDecimal(int, n) {
    int = int.padStart(n, '0');
    const part1 = int.slice(0, -n);
    const part2 = int.slice(-n);
    return part1 + '.' + part2;
}
exports.integerToDecimal = integerToDecimal;
